// Program "Bug8515.PAS" Copyright (C) 2000 B. Mueller (bmueller@mme-berlin.de) 
//
// This program forces the "Skip instruction with interrupts"-Error on the AT90S8515.
// It is written for the STK200 Starter-Kit from ATMEL. See the details to this serious
// bug in the ATMEL Errata-Sheet DOC1195.PDF.
//
// How does Bug8515 work?
// The AVR runs continuously through a critical code section, as described in the sheet.
// As long as the LEDs on the STK were steady shining, everything works fine, because
// the interrupts are disabled. The main loop of the program polls the Port-D keys and 
// enables the interrupts if any key has been pressed. This will directly crash the AVR.
//
// By using the argument $9509h for the critical sts instruction (which will be executed as
// an icall then), it is easy to demonstrate, that the AVR really has crashed. In this case,
// the LEDs on the STK are flashing. For details, see the code below ;-)
//
// Notice: The bug can be forced with any HLL (or even Assembler). It has *nothing* specific
// to do with Pascal.

link 0, $60 
program Bug8515;
var
   pBugHasOccured: Pointer;		// Pointer to the "crash-routine"

procedure StartUp;
// The start-up code initializes the interrupt vector table
begin
   asm
      rjmp @Bug8515
      rjmp EXT_INT0	; IRQ0 Handler
      rjmp EXT_INT1	; IRQ1 Handler
      rjmp TIM1_CAPT	; Timer1 Capture Handler
      rjmp TIM1_COMPA	; Timer1 CompareA Handler
      rjmp TIM1_COMPB	; Timer1 CompareB Handler
      rjmp TIM1_OVF	; Timer1 Overflow Handler
      rjmp @Bug8515|TIM0_OVF	; Timer0 Overflow Handler
      rjmp SPI_STC	; SPI Transfer Complete Handler
      rjmp UART_RXC	; UART RX Complete Handler
      rjmp UART_DRE	; UDR Empty Handler
      rjmp UART_TXC	; UART TX Complete Handler
      rjmp ANA_COMP	; Analog Comparator Handler
EXT_INT0:
EXT_INT1:
TIM1_CAPT:
TIM1_COMPA:
TIM1_COMPB:
TIM1_OVF:
SPI_STC:
UART_RXC:
UART_DRE:
UART_TXC:
ANA_COMP:
      reti
      Const    
      DS 0
      CodeSP
   end;
end;

procedure DelayALittleWhile;
var
   wCounter: Word;
begin
   for wCounter:= 1 to 15000 do;
end;

procedure BugHasOccured;
// If this procedure is entered, the AVR has crashed.
begin
   DI;					// No need for the interrupts any more 
   _PORTB:= $FF;			 
   repeat
      // Stay in loop and toogle the LEDs
      _PORTB:= _PORTB xor $FF;
      DelayALittleWhile;
   until FALSE;
end;

procedure TIM0_OVF; 			// Timer 0 interrupt procedure 
begin					// Just return
   asm
      reti
   end;
end;

begin
   _SPL:= lo(_RAMEND);			// Set the Stackpointer
   _SPH:= hi(_RAMEND);
   _DDRB:= $FF;  			// Switch Port B to output mode	
   _PORTD:= $FF;			// Switch Port D to input with Pull-Up
   _TCCR0:= 1;				// Clock 0 prescale select = CK
   _TIMSK:= __TOIE0;			// Enable Timer 0 interrupt
   _PORTB:= $FE;			// Port B initial value
   pBugHasOccured:= @BugHasOccured;
   asm
      .ldw Z, Bug8515|pBugHasOccured	;  Load the Z-register with the pointer  
   end;					// to the "crash-routine"		
   repeat
      _PORTB:= _PORTB RL 1;		// Let the LEDs shine a bit
      if IOBitLow(_SREG, $7) and (_PIND <> $FF) then begin
         // A key has been pressed, so enable the interrupts
         EI;
      end;
      // If the Timer interrupt occurs during the first or second clock-cycle of
      // the sbrs instruction, the AVR will decode the argument 9509h as an 
      // instruction and execute it. In our special case, the AVR will execute
      // an icall (which has the op-code 9509h) and call the "crash-routine"
      // indirect via the Z-register, which has been loaded before.
      asm
         ldi r20, 0ffh 
         sbrs r20, 7			
         sts 9509h, r20		
      end;       		
   until FALSE;			
end;